import pyb
 
data = "X8";    # patte connectée à l'entrée série du MAX7219 (DIN)
load = "X5";    # patte de chargement des données (CS)
clk  = "X6";    # patte donnant l'horloge de la liaison série (CLK)
 
# Initialisation des pattes en sortie en mode push-pull
dataPin = pyb.Pin(data, pyb.Pin.OUT_PP)
loadPin = pyb.Pin(load, pyb.Pin.OUT_PP)
clkPin  = pyb.Pin(clk, pyb.Pin.OUT_PP)
 
# Mise à zéro des pattes
dataPin.low()
loadPin.low()
clkPin.low()
 
# Transmet un octet bit par bit vers le MAX7219, bit de poids fort en premier
def serialShiftByte(data):
    # Mise à zéro du signal d'horloge pour pouvoir faire un front montant plus tard
    clkPin.low()
    # Décalage des 8 bits de données
    for i in range(8):
        # on décale la donnée de i bits vers la gauche et on teste le bit de poids fort
        value = ((data << i) & 0B10000000) != 0
        dataPin.value(value)  # on met la patte DIN à cette valeur
        clkPin.high()         # puis on crée une impulsion sur CLK
        clkPin.low()          # pour transmettre ce bit
 
# Écriture d'une donnée dans un registre du MAX7219.
def serialWrite(address, data):
    # Mise à zéro du signal CS pour pouvoir créer un front montant plus tard
    loadPin.low()
    # On envoie l'adresse en premier
    serialShiftByte(address)
    # puis la donnée
    serialShiftByte(data)
    # et on crée une impulsion sur la ligne CS pour charger la donnée dans le registre
    loadPin.high()
    loadPin.low()
    

def matrixOn(on):
    if on:
        serialWrite(0x0C,1)
    else:
        serialWrite(0x0C,0)
def matrixTest(test):
    if test:
        serialWrite(0x0F,1)
    else:
        serialWrite(0x0F,0)
        
def matrixIntensity(percent):
    serialWrite(0x0A,percent*16//100)

def matrixDecode(decode):
    if decode:
        serialWrite(0x09,0xFF)
    else:
        serialWrite(0x09,0x00)
def matrixDigits(num):
    serialWrite(0x0B,num)
    
def matrixLine(num,value):
    serialWrite(num,value)
    
bitmap = bytearray(8)

def updateDisplay(bitmap):
    for i in range(8):
        matrixLine(i+1,bitmap[i])
        
def clearDisplay(bitmap):
    for i in range(8):
        bitmap[i]=0
    updateDisplay(bitmap)

def setPixel(x,y,on,bitmap):
    if on:
        if (bitmap[x]%(pow(2,(8-y))))//(pow(2,(7-y)))==0:
            bitmap[x]+=pow(2,(7-y))
    else:
        if (bitmap[x]%(pow(2,(8-y))))//(pow(2,(7-y)))==1:
            bitmap[x]-=pow(2,(7-y))

def getPixel(x,y,bitmap):
    return (bitmap[x]%(pow(2,(8-y))))//(pow(2,(7-y)))

for i in range(8):
    bitmap[i]=i
updateDisplay(bitmap)
print(bitmap)
setPixel(5,5,False,bitmap)
print(bitmap)
updateDisplay(bitmap)





    
        
    

